@@ -19,6 +19,8 @@ namespace dxvk {
   
   class D3D11DeviceContext : public D3D11DeviceChild<ID3D11DeviceContext4> {
     friend class D3D11DeviceContextExt;
+    // Needed in order to call EmitCs for pushing markers
+    friend class D3D11UserDefinedAnnotation;
   public:
     
     D3D11DeviceContext(
@@ -778,17 +780,37 @@ namespace dxvk {
             D3D11UnorderedAccessView*         pUav,
             UINT                              CtrSlot,
             UINT                              Counter);
-    
+
+    void CopyBuffer(
+            D3D11Buffer*                      pDstBuffer,
+            VkDeviceSize                      DstOffset,
+            D3D11Buffer*                      pSrcBuffer,
+            VkDeviceSize                      SrcOffset,
+            VkDeviceSize                      ByteCount);
+
+    void CopyImage(
+            D3D11CommonTexture*               pDstTexture,
+      const VkImageSubresourceLayers*         pDstLayers,
+            VkOffset3D                        DstOffset,
+            D3D11CommonTexture*               pSrcTexture,
+      const VkImageSubresourceLayers*         pSrcLayers,
+            VkOffset3D                        SrcOffset,
+            VkExtent3D                        SrcExtent);
+
     void DiscardBuffer(
             ID3D11Resource*                   pResource);
     
     void DiscardTexture(
             ID3D11Resource*                   pResource,
             UINT                              Subresource);
-    
-    void SetIndexBufferOptimized(
-            BOOL                              Enable);
-    
+
+    void UpdateImage(
+            D3D11CommonTexture*               pDstTexture,
+      const VkImageSubresource*               pDstSubresource,
+            VkOffset3D                        DstOffset,
+            VkExtent3D                        DstExtent,
+            DxvkBufferSlice                   StagingBuffer);
+
     void SetDrawBuffers(
             ID3D11Buffer*                     pBufferForArgs,
             ID3D11Buffer*                     pBufferForCount);
@@ -830,7 +852,19 @@ namespace dxvk {
             ID3D11Buffer**                    ppConstantBuffers, 
             UINT*                             pFirstConstant, 
             UINT*                             pNumConstants);
-    
+
+    void GetShaderResources(
+      const D3D11ShaderResourceBindings&      Bindings,
+            UINT                              StartSlot,
+            UINT                              NumViews,
+            ID3D11ShaderResourceView**        ppShaderResourceViews);
+
+    void GetSamplers(
+      const D3D11SamplerBindings&             Bindings,
+            UINT                              StartSlot,
+            UINT                              NumSamplers,
+            ID3D11SamplerState**              ppSamplers);
+
     void ResetState();
 
     void RestoreState();
@@ -851,10 +885,6 @@ namespace dxvk {
     void RestoreUnorderedAccessViews(
             D3D11UnorderedAccessBindings&     Bindings);
     
-    void UpdateMappedBuffer(
-      const D3D11CommonTexture*               pTexture,
-            VkImageSubresource                Subresource);
-    
     bool TestRtvUavHazards(
             UINT                              NumRTVs,
             ID3D11RenderTargetView* const*    ppRTVs,
@@ -895,6 +925,9 @@ namespace dxvk {
     
     DxvkDataSlice AllocUpdateBufferSlice(size_t Size);
     
+    DxvkBufferSlice AllocStagingBuffer(
+            VkDeviceSize                      Size);
+    
     DxvkCsChunkRef AllocCsChunk();
     
     static void InitDefaultPrimitiveTopology(
