@@ -6,6 +6,7 @@
 
 #include "d3d11_context.h"
 #include "d3d11_state_object.h"
+#include "d3d11_video.h"
 
 namespace dxvk {
   
@@ -14,6 +15,7 @@ namespace dxvk {
   
   class D3D11ImmediateContext : public D3D11DeviceContext {
     friend class D3D11SwapChain;
+    friend class D3D11VideoContext;
   public:
     
     D3D11ImmediateContext(
@@ -21,6 +23,10 @@ namespace dxvk {
       const Rc<DxvkDevice>& Device);
     ~D3D11ImmediateContext();
     
+    HRESULT STDMETHODCALLTYPE QueryInterface(
+            REFIID  riid,
+            void**  ppvObject);
+
     D3D11_DEVICE_CONTEXT_TYPE STDMETHODCALLTYPE GetType();
     
     UINT STDMETHODCALLTYPE GetContextFlags();
@@ -118,6 +124,7 @@ namespace dxvk {
     dxvk::high_resolution_clock::time_point m_lastFlush
       = dxvk::high_resolution_clock::now();
     
+    D3D11VideoContext            m_videoContext;
     Com<D3D11DeviceContextState> m_stateObject;
     
     HRESULT MapBuffer(
