@@ -69,6 +69,10 @@ namespace dxvk {
             UINT                      PresentFlags,
       const DXGI_PRESENT_PARAMETERS*  pPresentParameters);
 
+    void STDMETHODCALLTYPE NotifyModeChange(
+            BOOL                      Windowed,
+      const DXGI_MODE_DESC*           pDisplayMode);
+
   private:
 
     enum BindingIds : uint32_t {
@@ -108,6 +112,8 @@ namespace dxvk {
     bool                    m_dirty = true;
     bool                    m_vsync = true;
 
+    double                  m_displayRefreshRate = 0.0;
+
     HRESULT PresentImage(UINT SyncInterval);
 
     void SubmitPresent(
@@ -134,7 +140,7 @@ namespace dxvk {
 
     void DestroyFrameLatencyEvent();
 
-    void SignalFrameLatencyEvent();
+    void SyncFrameLatency();
 
     uint32_t GetActualFrameLatency();
     
