@@ -223,7 +223,7 @@ namespace dxvk {
      */
     void DestroyBufferSubresource(UINT Subresource) {
       m_buffers[Subresource] = nullptr;
-      SetDirty(Subresource, true);
+      SetWrittenByGPU(Subresource, true);
     }
 
     bool IsDynamic() const {
@@ -308,11 +308,13 @@ namespace dxvk {
 
     bool GetLocked(UINT Subresource) const { return m_locked.get(Subresource); }
 
-    void SetDirty(UINT Subresource, bool value) { m_dirty.set(Subresource, value); }
+    bool IsAnySubresourceLocked() const { return m_locked.any(); }
 
-    bool GetDirty(UINT Subresource) const { return m_dirty.get(Subresource); }
+    void SetWrittenByGPU(UINT Subresource, bool value) { m_wasWrittenByGPU.set(Subresource, value); }
 
-    void MarkAllDirty() { m_dirty.setAll(); }
+    bool WasWrittenByGPU(UINT Subresource) const { return m_wasWrittenByGPU.get(Subresource); }
+
+    void MarkAllWrittenByGPU() { m_wasWrittenByGPU.setAll(); }
 
     void SetReadOnlyLocked(UINT Subresource, bool readOnly) { return m_readOnly.set(Subresource, readOnly); }
 
@@ -353,9 +355,14 @@ namespace dxvk {
     D3D9SubresourceBitset& GetUploadBitmask() { return m_needsUpload; }
 
     void SetNeedsUpload(UINT Subresource, bool upload) { m_needsUpload.set(Subresource, upload); }
-    bool GetNeedsUpload(UINT Subresource) const        { return m_needsUpload.get(Subresource); }
+    bool NeedsUpload(UINT Subresource) const { return m_needsUpload.get(Subresource); }
     bool NeedsAnyUpload() { return m_needsUpload.any(); }
     void ClearNeedsUpload() { return m_needsUpload.clearAll();  }
+    bool DoesStagingBufferUploads(UINT Subresource) const { return m_uploadUsingStaging.get(Subresource); }
+
+    void EnableStagingBufferUploads(UINT Subresource) {
+      m_uploadUsingStaging.set(Subresource, true);
+    }
 
     void SetNeedsMipGen(bool value) { m_needsMipGen = value; }
     bool NeedsMipGen() const { return m_needsMipGen; }
@@ -368,7 +375,7 @@ namespace dxvk {
     void PreLoadAll();
     void PreLoadSubresource(UINT Subresource);
 
-    void AddUpdateDirtyBox(CONST D3DBOX* pDirtyBox, uint32_t layer) {
+    void AddDirtyBox(CONST D3DBOX* pDirtyBox, uint32_t layer) {
       if (pDirtyBox) {
         D3DBOX box = *pDirtyBox;
         if (box.Right <= box.Left
@@ -376,32 +383,39 @@ namespace dxvk {
           || box.Back <= box.Front)
           return;
 
-        D3DBOX& updateBox = m_updateDirtyBoxes[layer];
-        if (updateBox.Left == updateBox.Right) {
-          updateBox = box;
+        box.Right = std::min(box.Right, m_desc.Width);
+        box.Bottom = std::min(box.Bottom, m_desc.Height);
+        box.Back = std::min(box.Back, m_desc.Depth);
+
+        D3DBOX& dirtyBox = m_dirtyBoxes[layer];
+        if (dirtyBox.Left == dirtyBox.Right) {
+          dirtyBox = box;
         } else {
-          updateBox.Left    = std::min(updateBox.Left,   box.Left);
-          updateBox.Right   = std::max(updateBox.Right,  box.Right);
-          updateBox.Top     = std::min(updateBox.Top,    box.Top);
-          updateBox.Bottom  = std::max(updateBox.Bottom, box.Bottom);
-          updateBox.Front   = std::min(updateBox.Front,  box.Front);
-          updateBox.Back    = std::max(updateBox.Back,   box.Back);
+          dirtyBox.Left    = std::min(dirtyBox.Left,   box.Left);
+          dirtyBox.Right   = std::max(dirtyBox.Right,  box.Right);
+          dirtyBox.Top     = std::min(dirtyBox.Top,    box.Top);
+          dirtyBox.Bottom  = std::max(dirtyBox.Bottom, box.Bottom);
+          dirtyBox.Front   = std::min(dirtyBox.Front,  box.Front);
+          dirtyBox.Back    = std::max(dirtyBox.Back,   box.Back);
         }
       } else {
-        m_updateDirtyBoxes[layer] = { 0, 0, m_desc.Width, m_desc.Height, 0, m_desc.Depth };
+        m_dirtyBoxes[layer] = { 0, 0, m_desc.Width, m_desc.Height, 0, m_desc.Depth };
       }
     }
 
-    void ClearUpdateDirtyBoxes() {
-      for (uint32_t i = 0; i < m_updateDirtyBoxes.size(); i++) {
-        m_updateDirtyBoxes[i] = { 0, 0, 0, 0, 0, 0 };
+    void ClearDirtyBoxes() {
+      for (uint32_t i = 0; i < m_dirtyBoxes.size(); i++) {
+        m_dirtyBoxes[i] = { 0, 0, 0, 0, 0, 0 };
       }
     }
 
-    const D3DBOX& GetUpdateDirtyBox(uint32_t layer) const {
-      return m_updateDirtyBoxes[layer];
+    const D3DBOX& GetDirtyBox(uint32_t layer) const {
+      return m_dirtyBoxes[layer];
     }
 
+    static VkImageType GetImageTypeFromResourceType(
+            D3DRESOURCETYPE  Dimension);
+
   private:
 
     D3D9DeviceEx*                 m_device;
@@ -432,17 +446,19 @@ namespace dxvk {
 
     D3D9SubresourceBitset         m_readOnly = { };
 
-    D3D9SubresourceBitset         m_dirty = { };
+    D3D9SubresourceBitset         m_wasWrittenByGPU = { };
 
     D3D9SubresourceBitset         m_needsUpload = { };
 
+    D3D9SubresourceBitset         m_uploadUsingStaging = { };
+
     DWORD                         m_exposedMipLevels = 0;
 
     bool                          m_needsMipGen = false;
 
     D3DTEXTUREFILTERTYPE          m_mipFilter = D3DTEXF_LINEAR;
 
-    std::array<D3DBOX, 6>         m_updateDirtyBoxes;
+    std::array<D3DBOX, 6>         m_dirtyBoxes;
 
     /**
      * \brief Mip level
@@ -477,9 +493,6 @@ namespace dxvk {
     VkImageLayout OptimizeLayout(
             VkImageUsageFlags         Usage) const;
 
-    static VkImageType GetImageTypeFromResourceType(
-            D3DRESOURCETYPE  Dimension);
-
     static VkImageViewType GetImageViewTypeFromResourceType(
             D3DRESOURCETYPE  Dimension,
             UINT             Layer);
