@@ -331,19 +331,15 @@ namespace dxvk {
       VK_FORMAT_UNDEFINED,
       VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_R8G8_B8G8_UNORM
-    { VK_FORMAT_B8G8R8G8_422_UNORM_KHR,
+    { VK_FORMAT_B8G8R8G8_422_UNORM,
       VK_FORMAT_UNDEFINED,
       VK_FORMAT_UNDEFINED,
-      VK_IMAGE_ASPECT_COLOR_BIT, 0,
-      { VK_COMPONENT_SWIZZLE_B, VK_COMPONENT_SWIZZLE_G,
-        VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_ONE }},
+      VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_G8R8_G8B8_UNORM
-    { VK_FORMAT_G8B8G8R8_422_UNORM_KHR,
+    { VK_FORMAT_G8B8G8R8_422_UNORM,
       VK_FORMAT_UNDEFINED,
       VK_FORMAT_UNDEFINED,
-      VK_IMAGE_ASPECT_COLOR_BIT, 0,
-      { VK_COMPONENT_SWIZZLE_B, VK_COMPONENT_SWIZZLE_G,
-        VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_ONE }},
+      VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_BC1_TYPELESS
     { VK_FORMAT_BC1_RGBA_UNORM_BLOCK,
       VK_FORMAT_UNDEFINED,
@@ -488,21 +484,35 @@ namespace dxvk {
       VK_FORMAT_UNDEFINED,
       VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_AYUV
-    { }, // Unsupported
+    { VK_FORMAT_R8G8B8A8_UNORM,
+      VK_FORMAT_UNDEFINED,
+      VK_FORMAT_R8G8B8A8_UINT,
+      VK_IMAGE_ASPECT_COLOR_BIT, 0,
+      { VK_COMPONENT_SWIZZLE_R, VK_COMPONENT_SWIZZLE_B,
+        VK_COMPONENT_SWIZZLE_G, VK_COMPONENT_SWIZZLE_A } },
     // DXGI_FORMAT_Y410
     { }, // Unsupported
     // DXGI_FORMAT_Y416
     { }, // Unsupported
     // DXGI_FORMAT_NV12
-    { }, // Unsupported
+    { VK_FORMAT_G8_B8R8_2PLANE_420_UNORM,
+      VK_FORMAT_UNDEFINED,
+      VK_FORMAT_UNDEFINED,
+      VK_IMAGE_ASPECT_PLANE_0_BIT | VK_IMAGE_ASPECT_PLANE_1_BIT },
     // DXGI_FORMAT_P010
     { }, // Unsupported
     // DXGI_FORMAT_P016
     { }, // Unsupported
     // DXGI_FORMAT_420_OPAQUE
-    { }, // Unsupported
+    { VK_FORMAT_G8_B8R8_2PLANE_420_UNORM,
+      VK_FORMAT_UNDEFINED,
+      VK_FORMAT_UNDEFINED,
+      VK_IMAGE_ASPECT_PLANE_0_BIT | VK_IMAGE_ASPECT_PLANE_1_BIT },
     // DXGI_FORMAT_YUY2
-    { }, // Unsupported
+    { VK_FORMAT_G8B8G8R8_422_UNORM,
+      VK_FORMAT_UNDEFINED,
+      VK_FORMAT_UNDEFINED,
+      VK_IMAGE_ASPECT_COLOR_BIT },
     // DXGI_FORMAT_Y210
     { }, // Unsupported
     // DXGI_FORMAT_Y216
@@ -787,21 +797,30 @@ namespace dxvk {
     { VK_FORMAT_BC7_UNORM_BLOCK,
       VK_FORMAT_BC7_SRGB_BLOCK },
     // DXGI_FORMAT_AYUV
-    { }, // Unsupported
+    { VK_FORMAT_R8G8B8A8_UNORM,
+      VK_FORMAT_R8G8B8A8_UINT },
     // DXGI_FORMAT_Y410
     { }, // Unsupported
     // DXGI_FORMAT_Y416
     { }, // Unsupported
     // DXGI_FORMAT_NV12
-    { }, // Unsupported
+    { VK_FORMAT_R8_UNORM,
+      VK_FORMAT_R8G8_UNORM,
+      VK_FORMAT_R8_UINT,
+      VK_FORMAT_R8G8_UINT },
     // DXGI_FORMAT_P010
     { }, // Unsupported
     // DXGI_FORMAT_P016
     { }, // Unsupported
     // DXGI_FORMAT_420_OPAQUE
-    { }, // Unsupported
+    { VK_FORMAT_R8_UNORM,
+      VK_FORMAT_R8G8_UNORM,
+      VK_FORMAT_R8_UINT,
+      VK_FORMAT_R8G8_UINT },
     // DXGI_FORMAT_YUY2
-    { }, // Unsupported
+    { VK_FORMAT_G8B8G8R8_422_UNORM,
+      VK_FORMAT_R8G8B8A8_UNORM,
+      VK_FORMAT_R8G8B8A8_UINT },
     // DXGI_FORMAT_Y210
     { }, // Unsupported
     // DXGI_FORMAT_Y216
