@@ -341,13 +341,15 @@ namespace dxvk {
     /**
      * \brief Copies data from a buffer to an image
      * 
+     * Source data must be packed, except for the row alignment.
      * \param [in] dstImage Destination image
      * \param [in] dstSubresource Destination subresource
      * \param [in] dstOffset Destination area offset
      * \param [in] dstExtent Destination area size
      * \param [in] srcBuffer Source buffer
      * \param [in] srcOffset Source offset, in bytes
-     * \param [in] srcExtent Source data extent
+     * \param [in] rowAlignment Row alignment, in bytes
+     * \param [in] sliceAlignment Slice alignment, in bytes
      */
     void copyBufferToImage(
       const Rc<DxvkImage>&        dstImage,
@@ -356,7 +358,8 @@ namespace dxvk {
             VkExtent3D            dstExtent,
       const Rc<DxvkBuffer>&       srcBuffer,
             VkDeviceSize          srcOffset,
-            VkExtent2D            srcExtent);
+            VkDeviceSize          rowAlignment,
+            VkDeviceSize          sliceAlignment);
     
     /**
      * \brief Copies data from one image to another
@@ -400,6 +403,8 @@ namespace dxvk {
      * \param [in] dstBuffer Destination buffer
      * \param [in] dstOffset Destination offset, in bytes
      * \param [in] dstExtent Destination data extent
+     * \param [in] rowAlignment Row alignment, in bytes
+     * \param [in] sliceAlignment Slice alignment, in bytes
      * \param [in] srcImage Source image
      * \param [in] srcSubresource Source subresource
      * \param [in] srcOffset Source area offset
@@ -408,7 +413,8 @@ namespace dxvk {
     void copyImageToBuffer(
       const Rc<DxvkBuffer>&       dstBuffer,
             VkDeviceSize          dstOffset,
-            VkExtent2D            dstExtent,
+            VkDeviceSize          rowAlignment,
+            VkDeviceSize          sliceAlignment,
       const Rc<DxvkImage>&        srcImage,
             VkImageSubresourceLayers srcSubresource,
             VkOffset3D            srcOffset,
@@ -422,7 +428,9 @@ namespace dxvk {
      * - \c VK_FORMAT_D24_UNORM_S8_UINT: 0xssdddddd
      * - \c VK_FORMAT_D32_SFLOAT_S8_UINT: 0xdddddddd 0x000000ss
      * \param [in] dstBuffer Destination buffer
-     * \param [in] dstOffset Destination offset, in bytes
+     * \param [in] dstBufferOffset Destination offset, in bytes
+     * \param [in] dstOffset Destination image offset
+     * \param [in] dstSize Destination image size
      * \param [in] srcImage Source image
      * \param [in] srcSubresource Source subresource
      * \param [in] srcOffset Source area offset
@@ -431,13 +439,43 @@ namespace dxvk {
      */
     void copyDepthStencilImageToPackedBuffer(
       const Rc<DxvkBuffer>&       dstBuffer,
-            VkDeviceSize          dstOffset,
+            VkDeviceSize          dstBufferOffset,
+            VkOffset2D            dstOffset,
+            VkExtent2D            dstExtent,
       const Rc<DxvkImage>&        srcImage,
             VkImageSubresourceLayers srcSubresource,
             VkOffset2D            srcOffset,
             VkExtent2D            srcExtent,
             VkFormat              format);
     
+    /**
+     * \brief Copies image data stored in a linear buffer to another
+     *
+     * The source and destination regions may overlap, in which case
+     * a temporary copy of the source buffer will be created.
+     * \param [in] dstBuffer Destination buffer
+     * \param [in] dstBufferOffset Destination subresource offset
+     * \param [in] dstOffset Destination image offset
+     * \param [in] dstSize Total size of the destination image
+     * \param [in] srcBuffer Source buffer
+     * \param [in] srcBufferOffset Source subresource offset
+     * \param [in] srcOffset Source image offset
+     * \param [in] srcSize Total size of the source image
+     * \param [in] extent Number of pixels to copy
+     * \param [in] elementSize Pixel size, in bytes
+     */
+    void copyPackedBufferImage(
+      const Rc<DxvkBuffer>&       dstBuffer,
+            VkDeviceSize          dstBufferOffset,
+            VkOffset3D            dstOffset,
+            VkExtent3D            dstSize,
+      const Rc<DxvkBuffer>&       srcBuffer,
+            VkDeviceSize          srcBufferOffset,
+            VkOffset3D            srcOffset,
+            VkExtent3D            srcSize,
+            VkExtent3D            extent,
+            VkDeviceSize          elementSize);
+
     /**
      * \brief Unpacks buffer data to a depth-stencil image
      * 
@@ -449,7 +487,9 @@ namespace dxvk {
      * \param [in] dstOffset Image area offset
      * \param [in] dstExtent Image area size
      * \param [in] srcBuffer Packed data buffer
-     * \param [in] srcOffset Packed data offset
+     * \param [in] srcBufferOffset Buffer offset of source image
+     * \param [in] srcOffset Offset into the source image
+     * \param [in] srcExtent Total size of the source image
      * \param [in] format Packed data format
      */
     void copyPackedBufferToDepthStencilImage(
@@ -458,7 +498,9 @@ namespace dxvk {
             VkOffset2D            dstOffset,
             VkExtent2D            dstExtent,
       const Rc<DxvkBuffer>&       srcBuffer,
-            VkDeviceSize          srcOffset,
+            VkDeviceSize          srcBufferOffset,
+            VkOffset2D            srcOffset,
+            VkExtent2D            srcExtent,
             VkFormat              format);
     
     /**
@@ -981,7 +1023,32 @@ namespace dxvk {
      * given context are rare.
      */
     void trimStagingBuffers();
-    
+   
+    /**
+     * \brief Begins a debug label region
+     * \param [in] label The debug label
+     *
+     * Marks the start of a debug label region. Used by debugging/profiling
+     * tools to mark different workloads within a frame.
+     */
+    void beginDebugLabel(VkDebugUtilsLabelEXT *label);
+
+    /**
+     * \brief Ends a debug label region
+     *
+     * Marks the close of a debug label region. Used by debugging/profiling
+     * tools to mark different workloads within a frame.
+     */
+    void endDebugLabel();
+
+    /**
+     * \brief Inserts a debug label
+     * \param [in] label The debug label
+     *
+     * Inserts an instantaneous debug label. Used by debugging/profiling
+     * tools to mark different workloads within a frame.
+     */
+    void insertDebugLabel(VkDebugUtilsLabelEXT *label);
   private:
     
     Rc<DxvkDevice>          m_device;
@@ -1036,6 +1103,28 @@ namespace dxvk {
       const VkImageBlit&          region,
             VkFilter              filter);
 
+    template<bool ToImage>
+    void copyImageBufferData(
+            DxvkCmdBuffer         cmd,
+      const Rc<DxvkImage>&        image,
+      const VkImageSubresourceLayers& imageSubresource,
+            VkOffset3D            imageOffset,
+            VkExtent3D            imageExtent,
+            VkImageLayout         imageLayout,
+      const DxvkBufferSliceHandle& bufferSlice,
+            VkDeviceSize          bufferRowAlignment,
+            VkDeviceSize          bufferSliceAlignment);
+
+    void copyImageHostData(
+            DxvkCmdBuffer         cmd,
+      const Rc<DxvkImage>&        image,
+      const VkImageSubresourceLayers& imageSubresource,
+            VkOffset3D            imageOffset,
+            VkExtent3D            imageExtent,
+      const void*                 hostData,
+            VkDeviceSize          rowPitch,
+            VkDeviceSize          slicePitch);
+
     void clearImageViewFb(
       const Rc<DxvkImageView>&    imageView,
             VkOffset3D            offset,
@@ -1067,6 +1156,14 @@ namespace dxvk {
             VkOffset3D            srcOffset,
             VkExtent3D            extent);
     
+    bool copyImageClear(
+      const Rc<DxvkImage>&        dstImage,
+            VkImageSubresourceLayers dstSubresource,
+            VkOffset3D            dstOffset,
+            VkExtent3D            dstExtent,
+      const Rc<DxvkImage>&        srcImage,
+            VkImageSubresourceLayers srcSubresource);
+
     void resolveImageHw(
       const Rc<DxvkImage>&            dstImage,
       const Rc<DxvkImage>&            srcImage,
@@ -1106,6 +1203,8 @@ namespace dxvk {
     void flushClears(
             bool                      useRenderPass);
 
+    void flushSharedImages();
+
     void startRenderPass();
     void spillRenderPass(bool suspend);
     
@@ -1238,4 +1337,4 @@ namespace dxvk {
 
   };
   
-}
\ No newline at end of file
+}
