@@ -278,7 +278,7 @@ namespace dxvk {
   
   
   DxvkRenderPass* DxvkRenderPassPool::getRenderPass(const DxvkRenderPassFormat& fmt) {
-    std::lock_guard<std::mutex> lock(m_mutex);
+    std::lock_guard<dxvk::mutex> lock(m_mutex);
 
     auto entry = m_renderPasses.find(fmt);
     if (entry != m_renderPasses.end())
