@@ -1,11 +1,11 @@
 #pragma once
 
 #include <atomic>
-#include <condition_variable>
-#include <mutex>
 
 #include "../rc/util_rc.h"
 
+#include "../thread.h"
+
 namespace dxvk::sync {
   
   /**
@@ -70,13 +70,13 @@ namespace dxvk::sync {
     }
 
     void signal(uint64_t value) {
-      std::unique_lock<std::mutex> lock(m_mutex);
+      std::unique_lock<dxvk::mutex> lock(m_mutex);
       m_value.store(value, std::memory_order_release);
       m_cond.notify_all();
     }
     
     void wait(uint64_t value) {
-      std::unique_lock<std::mutex> lock(m_mutex);
+      std::unique_lock<dxvk::mutex> lock(m_mutex);
       m_cond.wait(lock, [this, value] {
         return value <= m_value.load(std::memory_order_acquire);
       });
@@ -84,9 +84,9 @@ namespace dxvk::sync {
 
   private:
 
-    std::atomic<uint64_t>   m_value;
-    std::mutex              m_mutex;
-    std::condition_variable m_cond;
+    std::atomic<uint64_t>    m_value;
+    dxvk::mutex              m_mutex;
+    dxvk::condition_variable m_cond;
 
   };
   
